@@ -13,6 +13,7 @@
 
 #include <QImage>
 #include <QList>
+#include <QRegularExpression>
 
 #include <ksane_debug.h>
 
@@ -27,6 +28,8 @@
 #include "ksaneintegeroption.h"
 #include "ksaneinvertoption.h"
 #include "ksanepagesizeoption.h"
+#include "ksanebatchmodeoption.h"
+#include "ksanebatchdelayoption.h"
 
 namespace KSaneIface
 {
@@ -56,7 +59,9 @@ static const QHash<QString, KSaneCore::KSaneOptionName> stringEnumTranslation =
     { QStringLiteral(SANE_NAME_GAMMA_VECTOR_G), KSaneCore::GammaGreenOption },
     { QStringLiteral(SANE_NAME_GAMMA_VECTOR_B), KSaneCore::GammaBlueOption },
     { QStringLiteral(SANE_NAME_BLACK_LEVEL), KSaneCore::BlackLevelOption },
-    { QStringLiteral(SANE_NAME_WHITE_LEVEL), KSaneCore::WhiteLevelOption }, };
+    { QStringLiteral(SANE_NAME_WHITE_LEVEL), KSaneCore::WhiteLevelOption },
+    { BatchModeOptionName, KSaneCore::BatchModeOption },
+    { BatchDelayOptionName, KSaneCore::BatchDelayOption }, };
     
 KSaneCorePrivate::KSaneCorePrivate(KSaneCore *parent):
     q(parent)
@@ -70,6 +75,9 @@ KSaneCorePrivate::KSaneCorePrivate(KSaneCore *parent):
     m_auth = KSaneAuth::getInstance();
     m_optionPollTimer.setInterval(100);
     connect(&m_optionPollTimer, &QTimer::timeout, this, &KSaneCorePrivate::pollPollOptions);
+
+    m_batchModeTimer.setInterval(1000);
+    connect(&m_batchModeTimer, &QTimer::timeout, this, &KSaneCorePrivate::batchModeTimerUpdate);
 }
 
 KSaneCore::KSaneOpenStatus KSaneCorePrivate::loadDeviceOptions() 
@@ -106,7 +114,6 @@ KSaneCore::KSaneOpenStatus KSaneCorePrivate::loadDeviceOptions()
     }
     numSaneOptions = *reinterpret_cast<SANE_Word *>(data.data());
 
-    // read the rest of the options
     // read the rest of the options
     KSaneBaseOption *option = nullptr;
     KSaneBaseOption *optionTopLeftX = nullptr;
@@ -143,7 +150,8 @@ KSaneCore::KSaneOpenStatus KSaneCorePrivate::loadDeviceOptions()
             option = new KSaneActionOption(m_saneHandle, i);
             break;
         }
-
+        option->readOption();
+        option->readValue();
 
         if (option->name() == QStringLiteral(SANE_NAME_SCAN_TL_X)) {
             optionTopLeftX = option;
@@ -168,7 +176,7 @@ KSaneCore::KSaneOpenStatus KSaneCorePrivate::loadDeviceOptions()
         if (option->name() == QStringLiteral("wait-for-button")) {
             connect(option, &KSaneBaseOption::valueChanged, this, &KSaneCorePrivate::setWaitForExternalButton);
         }
-        
+
         m_optionsList.append(option);
         m_externalOptionsList.append(new KSaneInternalOption(option));
         connect(option, &KSaneBaseOption::optionsNeedReload, this, &KSaneCorePrivate::reloadOptions);
@@ -177,7 +185,7 @@ KSaneCore::KSaneOpenStatus KSaneCorePrivate::loadDeviceOptions()
         if (option->needsPolling()) {
             m_optionsPollList.append(option);
             if (option->type() == KSaneOption::TypeBool) {
-                connect( option, &KSaneBaseOption::valueChanged,
+                connect(option, &KSaneBaseOption::valueChanged, this,
                     [=]( const QVariant &newValue ) { Q_EMIT q->buttonPressed(option->name(), option->title(), newValue.toBool()); } );
             }
         }
@@ -186,35 +194,56 @@ KSaneCore::KSaneOpenStatus KSaneCorePrivate::loadDeviceOptions()
             m_optionsLocation.insert(it.value(), i - 1);
         }
     }
-    
-    // add extra option for inverting image colors
-    KSaneBaseOption *invertOption = new KSaneInvertOption();
-    m_optionsList.append(invertOption);
-    m_externalOptionsList.append(new KSaneInternalOption(invertOption));
-    m_optionsLocation.insert(KSaneCore::InvertColorOption, m_optionsList.size() - 1);  
+
     // add extra option for selecting specific page sizes
     KSaneBaseOption *pageSizeOption = new KSanePageSizeOption(optionTopLeftX, optionTopLeftY,
                             optionBottomRightX, optionBottomRightY, optionResolution);
     m_optionsList.append(pageSizeOption);
     m_externalOptionsList.append(new KSaneInternalOption(pageSizeOption));
-    m_optionsLocation.insert(KSaneCore::PageSizeOption, m_optionsList.size() - 1);  
-  
+    m_optionsLocation.insert(KSaneCore::PageSizeOption, m_optionsList.size() - 1);
+
+    // add extra option for batch mode scanning with a delay
+    m_batchMode = new KSaneBatchModeOption();
+    m_optionsList.append(m_batchMode);
+    m_externalOptionsList.append(new KSaneInternalOption(m_batchMode));
+    m_optionsLocation.insert(KSaneCore::BatchModeOption, m_optionsList.size() - 1);
+    m_batchModeDelay = new KSaneBatchDelayOption();
+    m_optionsList.append(m_batchModeDelay);
+    m_externalOptionsList.append(new KSaneInternalOption(m_batchModeDelay));
+    m_optionsLocation.insert(KSaneCore::BatchDelayOption, m_optionsList.size() - 1);
+
+    // add extra option for inverting image colors
+    KSaneBaseOption *invertOption = new KSaneInvertOption();
+    m_optionsList.append(invertOption);
+    m_externalOptionsList.append(new KSaneInternalOption(invertOption));
+    m_optionsLocation.insert(KSaneCore::InvertColorOption, m_optionsList.size() - 1);
+
+    // NOTICE The Pixma network backend behaves badly. polling a value will result in 1 second
+    // sleeps for every poll. The problem has been reported, but no easy/quick fix was available and
+    // the bug has been there for multiple years. Since this destroys the usability of the backend totally,
+    // we simply put the backend on the naughty list and disable the option polling.
+    static QRegularExpression pixmaNetworkBackend(QStringLiteral("pixma.*\\d+\\.\\d+\\.\\d+\\.\\d+"));
+    m_optionPollingNaughtylisted = false;
+    if (pixmaNetworkBackend.match(m_devName).hasMatch()) {
+        m_optionPollingNaughtylisted = true;
+    }
+
     // start polling the poll options
-    if (m_optionsPollList.size() > 0) {
+    if (m_optionsPollList.size() > 0 && !m_optionPollingNaughtylisted) {
         m_optionPollTimer.start();
     }
 
     // Create the scan thread
     m_scanThread = new KSaneScanThread(m_saneHandle);
-    
+
     m_scanThread->setImageInverted(invertOption->value());
     connect(invertOption, &KSaneInvertOption::valueChanged, m_scanThread, &KSaneScanThread::setImageInverted);
-    
+
     if (optionResolution != nullptr) {
         m_scanThread->setImageResolution(optionResolution->value());    
         connect(optionResolution, &KSaneBaseOption::valueChanged, m_scanThread, &KSaneScanThread::setImageResolution);
     }
-    
+
     connect(m_scanThread, &KSaneScanThread::scanProgressUpdated, q, &KSaneCore::scanProgress);
     connect(m_scanThread, &KSaneScanThread::finished, this, &KSaneCorePrivate::imageScanFinished);
 
@@ -230,7 +259,7 @@ void KSaneCorePrivate::clearDeviceOptions()
         delete m_optionsList.takeFirst();
         delete m_externalOptionsList.takeFirst();
     }
-    
+
     m_optionsLocation.clear();
     m_optionsPollList.clear();
     m_optionPollTimer.stop();
@@ -267,7 +296,7 @@ void KSaneCorePrivate::setDefaultValues()
 
     // Try to get Color mode by default
     if ((option = q->getOption(KSaneCore::ScanModeOption)) != nullptr) {
-        option->setValue(i18n(SANE_VALUE_SCAN_MODE_COLOR));
+        option->setValue(sane_i18n(SANE_VALUE_SCAN_MODE_COLOR));
     }
 
     // Try to set 8 bit color
@@ -311,18 +340,28 @@ void KSaneCorePrivate::pollPollOptions()
 
 void KSaneCorePrivate::imageScanFinished()
 {
+    Q_EMIT q->scanProgress(100);
     if (m_scanThread->frameStatus() == KSaneScanThread::ReadReady) {
         Q_EMIT q->scannedImageReady(*m_scanThread->scanImage());
         // now check if we should have automatic ADF batch scanning
         if (m_executeMultiPageScanning && !m_cancelMultiPageScan) {
             // in batch mode only one area can be scanned per page
+            Q_EMIT q->scanProgress(-1);
             m_scanThread->start();
             return;
         }
-
+        // check if we should have timed batch scanning
+        if (m_batchMode->value().toBool() && !m_cancelMultiPageScan) {
+            // in batch mode only one area can be scanned per page
+            m_batchModeCounter = 0;
+            batchModeTimerUpdate();
+            m_batchModeTimer.start();
+            return;
+        }
         // Check if we have a "wait for button" batch scanning
         if (m_waitForExternalButton) {
             qCDebug(KSANE_LOG) << "waiting for external button press to start next scan";
+            Q_EMIT q->scanProgress(-1);
             m_scanThread->start();
             return;
         }
@@ -356,7 +395,7 @@ void KSaneCorePrivate::imageScanFinished()
 void KSaneCorePrivate::scanIsFinished(KSaneCore::KSaneScanStatus status, const QString &message) 
 {
     sane_cancel(m_saneHandle);
-    if (m_optionsPollList.size() > 0) {
+    if (m_optionsPollList.size() > 0 && !m_optionPollingNaughtylisted) {
         m_optionPollTimer.start();
     }
 
@@ -378,4 +417,18 @@ void KSaneCorePrivate::setWaitForExternalButton(const QVariant &value)
     m_waitForExternalButton = value.toBool();
 }
 
+void KSaneCorePrivate::batchModeTimerUpdate()
+{
+    const int delay = m_batchModeDelay->value().toInt();
+    Q_EMIT q->batchModeCountDown(delay - m_batchModeCounter);
+    if (m_batchModeCounter >= delay) {
+        m_batchModeCounter = 0;
+        if (m_scanThread!= nullptr) {
+            m_scanThread->start();
+        }
+        m_batchModeTimer.stop();
+    }
+    m_batchModeCounter++;
+}
+
 }  // NameSpace KSaneIface
